(define (Sparse-matrix-Cholesky-factor m)
  (let* ((new-rows (list->vector (Sparse-matrix-rows m)))
	 (n (vector-length new-rows)))
    (do ((i 0 (fx+ i 1)))
	((fx= i n)
	 (instantiate Sparse-matrix
		      rows: (vector->list new-rows)
		      domain-dimension: (Sparse-matrix-domain-dimension m)
		      range-dimension: (Sparse-matrix-range-dimension m)))
      (let* ((row-i              (vector-ref new-rows i))
	     (diagonal-element   (car row-i))
	     (multiplier         (flsqrt (Entry-coefficient diagonal-element)))
	     (new-row-i          (Sparse-vector-scale (fl/ multiplier) row-i))
	     (new-row-i-tail     (cdr new-row-i)))
	(vector-set! new-rows i new-row-i)
	(for-each (lambda (entry)
		    (let* ((index (Entry-index entry))
			   (coeff (Entry-coefficient entry))
			   (row   (vector-ref new-rows index)))
		      (vector-set! new-rows
				   index
				   (Sparse-vector-*axpy (fl- coeff)
							new-row-i-tail
							(cdr row)))))
		  new-row-i-tail)))))

(define (Sparse-matrix-solve-Ux=b U b)
  (let* ((rows (list->vector (Sparse-matrix-rows U)))
	 (n (vector-length rows))
	 (result (f64vector-copy b)))
    (do ((i (fx- n 1) (fx- i 1)))
	((fx< i 0) result)
      (let* ((row (vector-ref rows i))
	     (diagonal-entry (car row))
	     (other-entries  (cdr row)))
	(for-each (lambda (entry)
		    (let ((index (Entry-index entry))
			  (coeff (Entry-coefficient entry)))
		      (f64vector-set! result i (fl- (f64vector-ref result i)
						    (fl* coeff (f64vector-ref result index))))))
		  other-entries)
	(f64vector-set! result i (fl/ (f64vector-ref result i)
				      (Entry-coefficient diagonal-entry)))))))

(define (Sparse-matrix-solve-U^Tx=b U b)
  (let* ((columns (list->vector (Sparse-matrix-rows U)))
	 (n (vector-length columns))
	 (result (f64vector-copy b)))
    (do ((i 0 (fx+ i 1)))
	((fx= i n) result)
      (let* ((column (vector-ref columns i))
	     (diagonal-entry (car column))
	     (other-entries  (cdr column)))
	(let ((ith-value (fl/ (f64vector-ref result i)
			      (Entry-coefficient diagonal-entry))))
	  (f64vector-set! result i ith-value)
	  (for-each (lambda (entry)
		      (let ((index (Entry-index entry))
			    (coeff (Entry-coefficient entry)))
			(f64vector-set! result index (fl- (f64vector-ref result index)
							  (fl* coeff ith-value)))))
		    other-entries))))))
